#include <qlabel.h>
#include <qpixmap.h>
#include <qlayout.h>

#include "devices.hh"
#include "ppp_oe_tab.hh"
#include "network_config.hh"
#include "pixmaps.hh"

InterfaceButton::InterfaceButton (QWidget* parent, QPixmap* picture, unsigned int id)
  : QPushButton (parent)
{
  setPixmap (*picture);
  setToggleButton (true);
  setMinimumHeight (50);
  my_id = id;
  QObject::connect (this, SIGNAL( toggled (bool) ), this, SLOT( selfToggled (bool) ));
  signal_lock = false;
}

void InterfaceButton::Up ()
{
  signal_lock = true;
  std::cout << my_id << " up" << std::endl;
  setOn(false);
  signal_lock = false;
}

void InterfaceButton::Down ()
{
  signal_lock = true;
  std::cout << my_id << " down" << std::endl;
  setOn(true);
  signal_lock = false;
}



void InterfaceButton::selfToggled (bool status)
{
  if (signal_lock)
    return;
  signal_lock = true;

  if (!status)
    setOn (true);
  else {
    emit PushedDown (my_id);
  }

  signal_lock = false;
}


PPPOETab::PPPOETab (QWidget* parent, const std::vector<EtherDev*>& devices)
  : QButtonGroup (parent), available_devices (devices)
{
  setColumnLayout (2, Qt::Horizontal);
  ether = ::GetEther ();
  wireless = ::GetWireless ();

  unsigned int device_count = available_devices.size();

  for (unsigned int i = 0; i < device_count; i += 2) {
    CreateInterfaceButton (i);
    if (i + 1 < device_count)
      CreateInterfaceButton (i+1);
    else // placeholder
      (new QLabel (this)) -> setText ("");
    CreateInterfaceLabel (i);
    if (i + 1 < device_count)
      CreateInterfaceLabel (i+1);
  }
}


void PPPOETab::NewConfig (network_config::PPPSubsection* subsection)
{
  config = subsection;
  // ensure some default values and put correct button down
  network_config::DeviceLine* dev = config -> Device ();
  int button_down = -1;
  for (unsigned int i = 0; i < available_devices.size (); i++) {
    if (dev -> device == available_devices[i] -> dev)
      button_down = 1;
  }

  if (button_down < 0) {
    button_down = 0;
    dev -> device = available_devices[0] -> dev;
  }
  dev -> speed = 0;
  dev -> Set ();

  config -> SetDefaults (true);
  config -> SetSpeedDefaults(true);
  config -> SetChatDefaults(false);
  network_config::OptionLine* logfile = config -> Option ("logfile", true);
  logfile -> value = "/var/log/ppp0"; // TODO: insert correct device here
  logfile -> Set ();

  SetInterface (button_down);
}


void PPPOETab::SetInterface (unsigned int id)
{
  for (unsigned int i = 0; i < buttons.size(); i++)
    if (i != id)
      buttons[i] -> Up ();
    else
      buttons[i] -> Down ();
  
  if (config) {
    config -> Device () -> device = available_devices[id] -> dev;
    config -> Device () -> Set ();
  }
}



void PPPOETab::CreateInterfaceButton (unsigned int i)
{
  buttons.push_back (new InterfaceButton (this, 
					  ((available_devices[i] -> wireless) ?
					    wireless : ether), i ));
  QObject::connect (buttons.back(), SIGNAL (PushedDown (unsigned int)), this, SLOT (InterfacePushed (unsigned int)));
}

void PPPOETab::CreateInterfaceLabel (unsigned int i)
{
  QLabel* l = new QLabel (this);
  l -> setText (available_devices[i] -> dev.c_str());
}


void PPPOETab::InterfacePushed (unsigned int id)
{
  std::cout << "pushed: " << id << std::endl;
  SetInterface (id);
}
