#include <fstream>
#include "devices.hh"

void DetectEtherDevices (EtherDevList& list);
void DetectSerialDevices (SerialDevList& list);
void ScanSerialDevices (SerialDevList& list); // trys to detect serpnp devices attached to the ports

// creates a list entry for given device file if it is not contained yet
void AddSerialUnique (SerialDevList& list, const std::string& device);

bool isWlan (EtherDev& dev);
std::string devInfo (EtherDev& dev);

int Execute (std::string& output_file, const std::string& cmd);
void CleanUp (const std::string& output_file);

// to ensure atomic operation and clean rewriting of active interfaces,
// a local copy of the config file is beeing used
bool ConfigfileHandleForReading (std::ifstream& stream);
bool ConfigfileHandleForWriting (std::ofstream& stream);
// copy local copy - make sure to close stream before that !
bool WriteConfig ();
void InitConfig ();

#ifdef PARALLEL_EXECUTION
class ParallelExecution
{
public:
  ParallelExecution (const std::string& cmd);
  ~ParallelExecution ();

  bool Finished ();
private:
  volatile bool finished;
  volatile bool retcode;
};

ParallelExecution* TriggerIfUp (const std::string& interface, const std::string& profile);
ParallelExecution* TriggerIfDown (const std::string& interface, const std::string& profile);
#endif

int TriggerIfUp (const std::string& interface, const std::string& profile);
int TriggerIfDown (const std::string& interface, const std::string& profile);

bool isUp (const std::string& interface, std::string* profile = 0); // writes the profilename to profile if not 0

int WirelessLinkQuality (const std::string& device);

// Todo : pppstatus functions

// returns first line of output or "" on failure / no output
std::string ExecuteSimple (const std::string& cmd);
